#lang racket
(require test-engine/racket-tests)

;; A dir is either 'too-big, 'too-small, or 'same

;; bsearch num num (num -> dir) -> num-or-false
;;  finds number between n and m inclusive for
;;  which guess returns 'same, or returns false
;;  if there's no such number
(define (bsearch n m guess)
  (cond
    [(= n m)
     (if (eq? (guess n) 'same)
         n
         false)]
    [else
     (define p (quotient (+ n m) 2))
     (define d (guess p))
     (cond
       [(eq? d 'too-big)
        (bsearch n p guess)]
       [(eq? d 'too-small)
        (bsearch (add1 p) m guess)]
       [else p])]))

;; make-reply : num -> (num -> dir)
(define (make-reply n)
  (lambda (v)
    (cond
      [(> v n) 'too-big]
      [(< v n) 'too-small]
      [else 'same])))

(check-expect (bsearch 0 10 (make-reply 5)) 5)
(check-expect (bsearch 6 10 (make-reply 5)) false)
(check-expect (bsearch 0 10 (make-reply 5.5)) false)

;; A listing is
;;  (make-listing string number)
(define-struct listing (name number))

;; The phonebook must be sorted by name:
(define phonebook
  (vector (make-listing "Chico" 1118888)
          (make-listing "Curly" 7778888)
          (make-listing "Groucho" 3338888)
          (make-listing "Gummo" 7779999)
          (make-listing "Harpo" 7770000)
          (make-listing "Larry" 4448888)
          (make-listing "Moe" 8887777)
          (make-listing "Zeppo" 2220000)))

;; lookup : string -> number-or-false
;;  Looks for the given name in `phonebook',
;;  returns the listing's number if found
(define (lookup name)
  (define pos 
    (bsearch 0 
             (sub1 (vector-length phonebook))
             (lambda (i)
               (define name2 (listing-name
                              (vector-ref phonebook i)))
               (cond
                 [(string>? name2 name) 'too-big]
                 [(string<? name2 name) 'too-small]
                 [else 'same]))))
  (if pos
      (listing-number (vector-ref phonebook pos))
      false))

(check-expect (lookup "Groucho") 3338888)
(check-expect (lookup "Chico") 1118888)
(check-expect (lookup "Zeppo") 2220000)
(check-expect (lookup "Krusty") false)

(test)
